//
// FullcalendarTimeline - Timeline manipulation controls
//

// Initialise timeline manipulation controls
FullcalendarTimeline.prototype.initialiseTimelineManipulationControls = function () {
  // Mechanism to pass (this) to the handlers
  var fullCalendarTimelineObject = this;

  this._timelineManipulationInProgress = false;

  this._zoomInButton = this.timelineViewWrapper.find('#zoom-in-button');
  this._zoomText = this.timelineViewWrapper.find('#zoom-text');
  this._zoomScaleSelector = this.timelineViewWrapper.find('#zoom-scale-selector');
  this._zoomOutButton = this.timelineViewWrapper.find('#zoom-out-button');

  this._zoomScales = ['day', 'week', 'month'];
  this._currentDurationCount = 1;
  this._currentDurationScale = 'day';
  this._visibleRangeStartDate = new Date();
  this._visibleRangeEndDate = dateAdd(this._visibleRangeStartDate, this._currentDurationCount, this._currentDurationScale);

  // Initialise drop-down list of the 'zoom-scale-selector'
  $.each(this._zoomScales, function (index, text) {
    fullCalendarTimelineObject._zoomScaleSelector.append(
      $('<option/>')
        .attr('value', text)
        .prop('selected', text == fullCalendarTimelineObject._currentDurationScale)
        .text(text)
    );
  });

  // Add event handlers to the 'zoom-scale-selector'
  this._zoomText.on('click', function () {
    $(this).attr('contentEditable', true);
  }).on('blur', function () {
    $(this).attr('contentEditable', false);
    fullCalendarTimelineObject._currentDurationCount = fullCalendarTimelineObject._zoomText.val();
    fullCalendarTimelineObject.updateTimelineDuration();
  });

  this._zoomText.on('change', function () {
    fullCalendarTimelineObject._currentDurationCount = fullCalendarTimelineObject._zoomText.val();
    fullCalendarTimelineObject.updateTimelineDuration();
  });

  this._zoomScaleSelector.on('change', function () {
    var newScale = fullCalendarTimelineObject._zoomScaleSelector.val();
    if (newScale != fullCalendarTimelineObject._currentDurationScale) {
      fullCalendarTimelineObject.updateTimelineDuration();
    }
  });

  this._zoomInButton.off('click').on('click', function () {
    fullCalendarTimelineObject.timelineZoomHandler('+');
  });

  this._zoomOutButton.off('click').on('click', function () {
    fullCalendarTimelineObject.timelineZoomHandler('-');
  });

  // Attach the date selector fullcalendarObject
  this.attachDatePickerToFullcalendarObject();
}

/**
 * Attach the date selector fullcalendarObject
 */
FullcalendarTimeline.prototype.attachDatePickerToFullcalendarObject = function () {
  // Mechanism to pass (this) to the handlers
  var fullCalendarTimelineObject = this;

  this.timelineElement.find('.fc-header-toolbar .fc-center h2').datetimepicker({
    timepicker: false,
    format: 'Y-m-d',
    onShow: function (ct) {
      this.setOptions({
        // TBD: Default to show the start date from the fullcalendarObject
        //startDate: fullCalendarTimelineObject._fullcalendarObject.getDate().yyyymmdd()
        //defaultDate: fullCalendarTimelineObject._fullcalendarObject.getDate().yyyymmdd()
      });
    },
    onChangeDateTime: function (dp, input) {
      fullCalendarTimelineObject.updateTimelineDuration(input.val());
    },
  });
}

/**
 * Re-render timeline view with the specified date range
 *
 * @param direction   '+' = increase date range / '-' = decrease date range
 */
FullcalendarTimeline.prototype.timelineZoomHandler = function (direction) {
  switch (direction) {
    case '+':
      this._currentDurationCount = this._currentDurationCount + 1;
      break;
    case '-':
      this._currentDurationCount = this._currentDurationCount - 1;
      break;
  }

  if (this._currentDurationCount < 1) { this._currentDurationCount = 1; }

  this.updateTimelineDuration();
}

/**
 * Re-render timeline view with the specified date range
 *
 * Examples:
 * updateTimelineDuration()
 * - use the date fields from the form
 *
 * updateTimelineDuration('2019-11-21')
 * - use the supplied start_date + the 'currentDurationCount'
 *
 * updateTimelineDuration('2019-11-21','2019-11-29')
 * - override any form values
 *
 * @param start_date Start date
 * @param end_date   End date
 */
FullcalendarTimeline.prototype.updateTimelineDuration = function (start_date, end_date) {

  // If start_date is not supplied as a parameter, use the reference date from the FullCalendar object
  if (typeof start_date === 'undefined' || start_date === null || start_date.length == 0) {
    start_date = this._fullcalendarObject.getDate().yyyymmdd();
  }

  // If we still have no 'start_date', get it from the DTSTART field
  // if (typeof start_date === 'undefined' || start_date === null || start_date.length == 0) {
  //   start_date = this._formData.start_date.substring(0, 10);
  // }

  var startDate = new Date(start_date);

  this._currentDurationScale = this._zoomScaleSelector.val();

  // var conversionDivisor;
  // switch (this._currentDurationScale) {
  //   case 'month':
  //     // TODO: Fix me
  //     // 1 month (28 days) = 2419200000 ms
  //     // 1 month (29 days) = 2505600000 ms
  //     // 1 month (30 days) = 2592000000 ms
  //     // 1 month (31 days) = 2678400000 ms
  //     conversionDivisor = (1000 * 60 * 60 * 24 * 30);
  //     break;
  //   case 'week':
  //     // 1 week = 604800000 ms
  //     conversionDivisor = (1000 * 60 * 60 * 24 * 7);
  //     break;
  //   case 'day':
  //   default:
  //     // 1 day =  86400000 ms
  //     conversionDivisor = (1000 * 60 * 60 * 24);
  // }

  // If end_date is not supplied as a parameter, calculate it from the 'zoom-controls'
  if (typeof end_date === 'undefined' || end_date === null || end_date.length == 0) {
    end_date = dateAdd(startDate, this._currentDurationCount - 1, this._currentDurationScale).yyyymmdd();
  }

  var endDate = new Date(end_date);

  // this._currentDurationCount = Math.round((endDate - startDate) / conversionDivisor) + 1;
  this._currentDurationCount = moment(endDate).diff(moment(startDate), this._currentDurationScale) + 2;
  // TODO: This should be +1. Find the root cause of why the final repeat day is sometimes lost

  if (this._currentDurationCount < 1) { this._currentDurationCount = 1; }

  // Update the fullcalendarObject
  this._visibleRangeStartDate = new Date(startDate);
  this._visibleRangeEndDate = dateAdd(startDate, this._currentDurationCount, this._currentDurationScale);

  this.fullcalendarObject.changeView('resourceTimeline', {
    start: this._visibleRangeStartDate,
    end: this._visibleRangeEndDate,
  });

  this.fullcalendarObject.setOption('duration', {
    days: Math.abs(this._visibleRangeStartDate.diff(this._visibleRangeEndDate, 'day'))
  });

  this.fullcalendarObject.gotoDate(this._visibleRangeStartDate);

  /******************************************************************************************/
  // The following is to setup the highlighterbar to be always present (no scrolling is required to see it).
  // The start of the timeline view starts 2 hours prior to the start time of the first highlighterbar.
  var newTimelineViewStart = moment(startDate).subtract(2, 'hours'); // Subtract 2 Hours from the start time of the first highlighterbar
  var hours = new Date(newTimelineViewStart).getHours().toString();
  if (hours == "0") {
    hours = "00";
  }
  var minute = new Date(newTimelineViewStart).getMinutes().toString();
  if (minute == "0") {
    minute = "00";
  }
  var newTime = hours + ":" + minute;
  this.fullcalendarObject.setOption('scrollTime', newTime);
/******************************************************************************************/

  // Attach top-scroll bars
  this.renderTopScrollBar();

  // Attach the date selector to the fullcalendarObject
  this.attachDatePickerToFullcalendarObject();

  // Update zoom-text
  this._zoomText.val(this._currentDurationCount);
}
