require "resource_planner/engine"

module ResourcePlanner
  # Your code goes here...

  # The parent controller all engine controllers inherits from.
  # Defaults to ApplicationController. This should be set early
  # in the initialisation process and should be set to a string.
  mattr_accessor :parent_controller
  self.parent_controller = "ApplicationController"

  # Our host application root path
  # We set this when the engine is initialised
  # mattr_accessor :app_root
  # @@app_root = 'xyz'

  # Allow clients to register application preferences.
  #
  # Clients may register additional models like this:
  #   application_preferences << { key: :key_name, data_query: '<ActiveRecord query>' }
  #
  @@application_preferences ||= []
  mattr_reader :application_preferences

  # Allow clients to register a parent "Event" class which will be used to
  # extract application related fields.
  #
  # :class
  # The fully qualified name of the model to which this definition applies
  #
  # :description_field
  # If :description_field contains an existing column, the column will be
  #
  # Clients may register a parent "Event" class like this:
  #   event_class = { :class => 'Model',
  #                   :description_field => :comment
  #                 }
  #
  @@event_class ||= {}
  mattr_accessor :event_class

  # Allow clients to register resource models.
  #
  # Clients may register additional models like this:
  #   resource_models << { model_reference: 'reference_name', model: 'ModelName', data_query: 'ModelName.all', resource_group_name: nil }
  #   resource_models << { model_reference: :reference_name, model: 'ModelName', data_query: nil }
  #   resource_models << { model_reference: :reference_name, model: 'ModelName', data_query: '', resource_group_name: nil }
  #   resource_models << { model_reference: :reference_name, model: 'ModelName', data_query: '', resource_group_name: 'Some arbitrary title' }
  #   resource_models << { model_reference: :reference_name, model: 'ModelName', resource_group_name_attribute: 'attribute.method' }
  #
  # PARAMETER NOTES
  #   model_reference: A name to uniquely identify the resource model definition (string or symbol)
  #                    (see resource_model_groups)
  #
  #   model: ActiveRecord model from which the resource data will come from
  #          It must implement 2 methods:
  #            to_s
  #            quantity
  #
  #   data_query: If nil or blank, then ModelName.all will be used to get the resource records
  #               If populated, the value will be treated as an ActiveRecord query to get the resource records
  #
  #   resource_group_name: If nil or blank, then 'ModelName'.pluralise will be used as the resource group name
  #                        If populated, the value will be used as the resource group name
  #
  #   resource_group_name_attribute: If nil or blank, see resource_group_name
  #                                  If populated, the value will be treated as an ActiveRecord field to get the resource_group_name from the record
  #
  @@resource_models ||= []
  mattr_reader :resource_models

  # Override the resource_models  (<<)instance method, to extend all defined model(s):
  class << resource_models
    def << value
      begin
        value[:model].constantize.class_eval do
          has_many :subscriptions, :as => :resource, :class_name => '::ResourcePlanner::EventUnitSubscription'
          has_many :event_units, :through => :subscriptions, :class_name => '::ResourcePlanner::EventUnit'

          has_many :availabilty_date_definitions, :as => :resource, :class_name => '::ResourcePlanner::AvailabiltyDateDefinition'
          has_many :availabilty_dates, :through => :availabilty_date_definitions, :class_name => '::ResourcePlanner::EventUnit'

          has_many :availabilty_restriction_definitions, :as => :resource, :class_name => '::ResourcePlanner::AvailabiltyRestrictionDefinition'
          has_many :availabilty_restrictions, :through => :availabilty_restriction_definitions, :class_name => '::ResourcePlanner::EventUnit'
        end

        self.push( value )
      rescue Exception => e
        puts e.message
        # e.backtrace.inspect.each { |line| puts line }
      end
    end
  end

  # Allow clients to register groups of resource models
  #
  # Clients may register resource model groups like this:
  #   resource_model_groups << { name => ['model_reference1', 'model_reference2', ...] }
  #   resource_model_groups << { name => [:model_reference1, :model_reference2, ...] }
  #
  # TODO: Update to be aligned with the updated resource_models structure
  #
  @@resource_model_groups ||= []
  mattr_reader :resource_model_groups

  # Default way to setup the engine.
  #
  # Run 'rails generate accounts_management_install' to create a fresh initialiser with all
  # configuration values.
  def self.setup
    yield self
  end

  def self.railtie_name
    ::ResourcePlanner::Engine.railtie_name
  end

  def self.mounted_path
    ::ResourcePlanner::Engine.mounted_path
  end

  def self.root_path
    ::ResourcePlanner::Engine.root
  end

end

# Require our engine
require 'resource_planner/version'
require 'resource_planner/engine'
